@@ -1,5 +1,7 @@
 /* cygwait.h
 
+   Copyright 2011, 2012, 2013, 2015 Red Hat, Inc.
+
    This file is part of Cygwin.
 
    This software is a copyrighted work licensed under the terms of the
@@ -39,8 +41,7 @@ cygwait (HANDLE object, PLARGE_INTEGER timeout, unsigned mask)
     wait_objects[num++] = object;
 
   wait_signal_arrived thread_waiting (is_cw_sig_handle, wait_objects[num]);
-  debug_only_printf ("object %p, thread waiting %d, signal_arrived %p",
-		     object, (int) thread_waiting, _my_tls.signal_arrived);
+  debug_only_printf ("object %p, thread waiting %d, signal_arrived %p", object, (int) thread_waiting, _my_tls.signal_arrived);
   DWORD sig_n;
   if (!thread_waiting)
     sig_n = WAIT_TIMEOUT + 1;
@@ -58,20 +59,16 @@ cygwait (HANDLE object, PLARGE_INTEGER timeout, unsigned mask)
     }
 
   DWORD timeout_n;
-  HANDLE local_timer = NULL;
-  HANDLE &wait_timer =
-    _my_tls.locals.cw_timer_inuse ? local_timer : _my_tls.locals.cw_timer;
   if (!timeout)
     timeout_n = WAIT_TIMEOUT + 1;
   else
     {
-      if (!_my_tls.locals.cw_timer_inuse)
-	_my_tls.locals.cw_timer_inuse = true;
       timeout_n = WAIT_OBJECT_0 + num++;
-      if (!wait_timer)
-	NtCreateTimer (&wait_timer, TIMER_ALL_ACCESS, NULL, NotificationTimer);
-      NtSetTimer (wait_timer, timeout, NULL, NULL, FALSE, 0, NULL);
-      wait_objects[timeout_n] = wait_timer;
+      if (!_my_tls.locals.cw_timer)
+	NtCreateTimer (&_my_tls.locals.cw_timer, TIMER_ALL_ACCESS, NULL,
+		       NotificationTimer);
+      NtSetTimer (_my_tls.locals.cw_timer, timeout, NULL, NULL, FALSE, 0, NULL);
+      wait_objects[timeout_n] = _my_tls.locals.cw_timer;
     }
 
   while (1)
@@ -104,19 +101,13 @@ cygwait (HANDLE object, PLARGE_INTEGER timeout, unsigned mask)
     {
       TIMER_BASIC_INFORMATION tbi;
 
-      NtQueryTimer (wait_timer, TimerBasicInformation, &tbi,
+      NtQueryTimer (_my_tls.locals.cw_timer, TimerBasicInformation, &tbi,
 		    sizeof tbi, NULL);
       /* if timer expired, TimeRemaining is negative and represents the
 	  system uptime when signalled */
-      if (timeout->QuadPart < 0LL) {
-	timeout->QuadPart = tbi.SignalState || tbi.TimeRemaining.QuadPart < 0LL
-                            ? 0LL : tbi.TimeRemaining.QuadPart;
-      }
-      NtCancelTimer (wait_timer, NULL);
-      if (local_timer)
-	NtClose(local_timer);
-      else
-	_my_tls.locals.cw_timer_inuse = false;
+      if (timeout->QuadPart < 0LL)
+	timeout->QuadPart = tbi.SignalState ? 0LL : tbi.TimeRemaining.QuadPart;
+      NtCancelTimer (_my_tls.locals.cw_timer, NULL);
     }
 
   if (res == WAIT_CANCELED && is_cw_cancel_self)
